import React, { useState, useEffect, useRef } from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  scaleLinear,
  scaleTime,
  axisLeft,
  axisBottom,
  select,
  max,
  timeFormat,
} from 'd3';

const csvUrl =
  'https://gist.githubusercontent.com/Alasrar/e99c767637381cf5681a7ed40bd2835f/raw/0e2fa6baf542c86e604df75e191b835bbc769938/Cancer_in_USA.csv';

const cancerTypes = [
  { type: 'Liver cancer', color: 'steelblue' },
  { type: 'Kidney cancer', color: 'tomato' },
  { type: 'Breast cancer', color: 'pink' },
  { type: 'Lip and oral cavity cancer', color: 'green' },
  { type: 'Larynx cancer', color: 'purple' },
  {
    type: 'Esophageal cancer',
    color: 'orange',
  },
  { type: 'Skin melanoma', color: 'deepskyblue' },
  { type: 'Leukemia', color: 'gold' },
  { type: 'Hodgkin lymphoma', color: 'limegreen' },
  { type: 'Prostate cancer', color: 'royalblue' },
  { type: 'Thyroid cancer', color: 'violet' },
  { type: 'Stomach cancer', color: 'sienna' },
  { type: 'Bladder cancer', color: 'teal' },
  { type: 'Uterine cancer', color: 'coral' },
  { type: 'Ovarian cancer', color: 'mediumorchid' },
];

const App = () => {
  const [data, setData] = useState([]);
  const svgRef = useRef();

  useEffect(() => {
    csv(csvUrl).then((loadedData) => {
      const processedData = loadedData
        .filter((d) => d.Code === 'USA')
        .map((d) => {
          const year = new Date(+d.Year, 0, 1);
          const deathsByCancer = cancerTypes.reduce(
            (acc, cancer) => {
              acc[cancer.type] =
                +d[cancer.type.replace(',', '')];
              return acc;
            },
            {},
          );
          return { year, ...deathsByCancer };
        });
      setData(processedData);
    });
  }, []);

  useEffect(() => {
    if (data.length > 0) {
      const svg = select(svgRef.current);
      const width = 960;
      const height = 500;
      const margin = {
        top: 50,
        right: 20,
        bottom: 50,
        left: 70,
      };
      const innerWidth = width - margin.left - margin.right;
      const innerHeight =
        height - margin.top - margin.bottom;
      const xScale = scaleTime()
        .domain([
          new Date(1990, 0, 1),
          new Date(2019, 11, 31),
        ])
        .range([0, innerWidth]);
      const yScale = scaleLinear()
        .domain([
          0,
          max(
            data.flatMap((d) => Object.values(d).slice(1)),
          ),
        ])
        .nice()
        .range([innerHeight, 0]);

      svg.selectAll('*').remove();
      const g = svg
        .attr('width', width)
        .attr('height', height)
        .append('g')
        .attr(
          'transform',
          `translate(${margin.left},${margin.top})`,
        );

      g.append('g').call(axisLeft(yScale));
      g.append('g')
        .attr('transform', `translate(0,${innerHeight})`)
        .call(axisBottom(xScale))
        .append('text')
        .attr('fill', '#000')
        .attr('x', innerWidth / 2)
        .attr('y', 45)
        .style('text-anchor', 'middle')
        .style('font-weight', 'bold')
        .text('Year');

      const tooltip = select('body')
        .append('div')
        .style('position', 'absolute')
        .style('background', 'rgba(255, 255, 255, 0.8)')
        .style('padding', '5px')
        .style('border', '1px solid #000')
        .style('border-radius', '5px')
        .style('visibility', 'hidden')
        .style('pointer-events', 'none')
        .style('font-weight', 'bold');

      cancerTypes.forEach((cancer) => {
        data.forEach((d) => {
          if (d[cancer.type]) {
            g.append('circle')
              .attr('cx', xScale(d.year))
              .attr('cy', yScale(d[cancer.type]))
              .attr('r', 3.5)
              .attr('fill', cancer.color)
              .on('mouseover', (event) => {
                tooltip
                  .style('visibility', 'visible')
                  .style('background', cancer.color)
                  .style('color', '#fff')
                  .style(
                    'border',
                    `1px solid ${cancer.color}`,
                  )
                  .html(
                    `${cancer.type}: ${d[cancer.type]} deaths in ${timeFormat('%Y')(d.year)}`,
                  )
                  .style('top', `${event.pageY - 10}px`)
                  .style('left', `${event.pageX + 10}px`);
              })
              .on('mouseout', () => {
                tooltip.style('visibility', 'hidden');
              });
          }
        });
      });

      const itemsPerRow = 4;
      const legendSpacing = 150;
      const legendRowHeight = 20;
      const legendXStart =
        (width - itemsPerRow * legendSpacing) / 2;

      const legend = svg
        .append('g')
        .attr(
          'transform',
          `translate(${legendXStart},${margin.top / 2})`,
        );

      cancerTypes.forEach((cancer, index) => {
        const row = Math.floor(index / itemsPerRow);
        const col = index % itemsPerRow;

        const legendItem = legend
          .append('g')
          .attr(
            'transform',
            `translate(${col * legendSpacing}, ${row * legendRowHeight})`,
          );

        legendItem
          .append('rect')
          .attr('width', 10)
          .attr('height', 10)
          .attr('fill', cancer.color);

        legendItem
          .append('text')
          .attr('x', 15)
          .attr('y', 10)
          .style('text-anchor', 'start')
          .style('text-transform', 'capitalize')
          .text(cancer.type);
      });
    }
  }, [data]);

  return <svg ref={svgRef}></svg>;
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
